@@ -23,6 +23,8 @@ import Test.QuickCheck.Instances ()
 
 import Control.Applicative (Applicative(..), liftA2)
 import Data.Functor ((<$>))
+import Data.Functor.Const (Const(..))
+import Data.Functor.Identity (Identity(..))
 import Data.Foldable (foldMap, toList)
 import Data.Int (Int8, Int32)
 import qualified Data.Foldable as Foldable
@@ -31,6 +33,7 @@ import Data.List (intersperse, unfoldr)
 import qualified Data.List as List
 import Data.Maybe (isJust)
 import Data.Either (lefts, rights)
+import Data.Proxy (Proxy)
 import Data.Tuple (swap)
 import Data.String (IsString, fromString)
 import Data.Char (isLetter)
@@ -64,7 +67,7 @@ import qualified Data.Monoid.Instances.Stateful as Stateful
 import Data.Monoid.Instances.Positioned (OffsetPositioned, LinePositioned)
 import qualified Data.Monoid.Instances.Positioned as Positioned
 
-import Data.Semigroup (Semigroup, (<>))
+import Data.Semigroup (Semigroup, (<>), Max, Min)
 import Data.Monoid (Monoid, mempty, mconcat, All(All), Any(Any), Dual(Dual),
                     First(First), Last(Last), Sum(Sum), Product(Product))
 import Data.Semigroup.Factorial (Factorial, StableFactorial, 
@@ -206,6 +209,11 @@ positiveInstances = map upcast stableFactorialInstances
                          PositiveMonoidInstance (mempty :: Ordering),
                          PositiveMonoidInstance (mempty :: All),
                          PositiveMonoidInstance (mempty :: Any),
+                         PositiveMonoidInstance (mempty :: Max Int),
+                         PositiveMonoidInstance (mempty :: Min Int),
+                         PositiveMonoidInstance (mempty :: Const String Float),
+                         PositiveMonoidInstance (mempty :: Identity Ordering),
+                         PositiveMonoidInstance (mempty :: (Proxy Float)),
                          PositiveMonoidInstance (mempty :: (Maybe (Sum Integer))),
                          PositiveMonoidInstance (mempty :: (Product Natural)),
                          PositiveMonoidInstance (mempty :: (Sum Natural)),
@@ -317,6 +325,12 @@ rightReductiveInstances = map upcast rightCancellativeInstances
 
 reductiveInstances = map upcast cancellativeInstances
                      ++ [ReductiveMonoidInstance (mempty :: Product Integer),
+                         ReductiveMonoidInstance (mempty :: Max Int),
+                         ReductiveMonoidInstance (mempty :: Min Int),
+                         ReductiveMonoidInstance (mempty :: All),
+                         ReductiveMonoidInstance (mempty :: Any),
+                         ReductiveMonoidInstance (mempty :: Identity IntSet),
+                         ReductiveMonoidInstance (mempty :: Const (Sum Int) Float),
                          ReductiveMonoidInstance (mempty :: IntSet),
                          ReductiveMonoidInstance (mempty :: Maybe IntSet),
                          ReductiveMonoidInstance (mempty :: Set Integer)]
